/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__TERMINAL_H__)
#define __TERMINAL_H__


/* determine the type of terminal interface */
#if !defined(USE_TERMIOS)
#if !defined(USE_TERMIO)
#if !defined(USE_SGTTY)
/* use termios */
#define USE_TERMIOS  /* <termios.h> */
#undef USE_TERMIO    /* <termio.h> */
#undef USE_SGTTY     /* <sys/ioctl.h> */
#endif
#endif
#endif


#endif /* !__TERMINAL_H__ */
