/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_RUSAGE_H__)
#define __HAVE_RUSAGE_H__


/* do we have or want getrusage()? */
#define HAVE_GETRUSAGE /* yes */


#endif /* !__HAVE_RUSAGE_H__ */
