/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_GETPGID_H__)
#define __HAVE_GETPGID_H__


/* do we have or want getpgid()? */
#undef HAVE_GETPGID /* no */


#endif /* !__HAVE_GETPGID_H__ */
