/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_UID_T_H__)
#define __HAVE_UID_T_H__


/* do we have or want uid_t? */
#define HAVE_UID_T /* yes */


#endif /* !__HAVE_UID_T_H__ */
