/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_UNISTD_H__)
#define __HAVE_UNISTD_H__


/* do we have /usr/include/unistd.h? */
#define HAVE_UNISTD_H	 /* yes */


#endif /* !__HAVE_UNISTD_H__ */
