/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_USTAT_H__)
#define __HAVE_USTAT_H__


/* do we have or want ustat()? */
#undef HAVE_USTAT /* no */


#endif /* !__HAVE_USTAT_H__ */
