/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__CONF_H__)
#define __CONF_H__


/* the default :-separated search path */
#if !defined(DEFAULTCALCPATH)
#define DEFAULTCALCPATH ".:./cal:~/.cal:/usr/local/share/calc:/usr/local/share/calc/custom"
#endif /* DEFAULTCALCPATH */

/* the default :-separated startup file list */
#if !defined(DEFAULTCALCRC)
#define DEFAULTCALCRC "/usr/local/share/calc/startup:~/.calcrc:./.calcinit"
#endif /* DEFAULTCALCRC */

/* the location of the help directory */
#if !defined(HELPDIR)
#define HELPDIR "/usr/local/share/calc/help"
#endif /* HELPDIR */

/* the location of the custom help directory */
#if !defined(CUSTOMHELPDIR)
#define CUSTOMHELPDIR "/usr/local/share/calc/custhelp"
#endif /* CUSTOMHELPDIR */

/* the default pager to use */
#if !defined(DEFAULTCALCPAGER)
#define DEFAULTCALCPAGER "more"
#endif /* DEFAULTCALCPAGER */

/* where the echo command is located */
#if !defined(ECHO_PROG)
#define ECHO_PROG "/bin/echo"
#endif /* ECHO_PROG */


#endif /* !__CONF_H__ */
