/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_NEWSTR_H__)
#define __HAVE_NEWSTR_H__


/* do we have or want memcpy(), memset() & strchr()? */
#define HAVE_NEWSTR /* yes */


#endif /* !__HAVE_NEWSTR_H__ */
