/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_GETPRID_H__)
#define __HAVE_GETPRID_H__


/* do we have or want getprid()? */
#undef HAVE_GETPRID /* no */


#endif /* !__HAVE_GETPRID_H__ */
