/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_CONST_H__)
#define __HAVE_CONST_H__


/* do we have or want const? */
#define HAVE_CONST /* yes */
#undef CONST
#define CONST const /* yes */


#endif /* !__HAVE_CONST_H__ */
