/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_STRING_H__)
#define __HAVE_STRING_H__


/* do we have /usr/include/string.h? */
#define HAVE_STRING_H	 /* yes */


#endif /* !__HAVE_STRING_H__ */
