/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_TIMES_H__)
#define __HAVE_TIMES_H__


/* do we have /usr/include/times.h? */
#undef HAVE_TIMES_H  /* no */
#define HAVE_SYS_TIMES_H  /* yes */
#define HAVE_TIME_H  /* yes */
#define HAVE_SYS_TIME_H  /* yes */


#endif /* !__HAVE_TIMES_H__ */
