/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__ENDIAN_CALC_H__)
#define __ENDIAN_CALC_H__


/* what byte order are we? */
#include <machine/endian.h>
#define CALC_BYTE_ORDER BYTE_ORDER


#endif /* !__ENDIAN_CALC_H__ */
