/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_POSSCL_H__)
#define __HAVE_POSSCL_H__


#undef HAVE_FILEPOS_SCALAR
#define HAVE_FILEPOS_SCALAR /* FILEPOS is a simple value */


#endif /* !__HAVE_POSSCL_H__ */
