/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_MALLOC_H__)
#define __HAVE_MALLOC_H__


/* do we have /usr/include/malloc.h? */
#define HAVE_MALLOC_H	 /* yes */


#endif /* !__HAVE_MALLOC_H__ */
