/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_MEMMV_H__)
#define __HAVE_MEMMV_H__


/* do we have or want memmove()? */
#define HAVE_MEMMOVE /* yes */


#endif /* !__HAVE_MEMMV_H__ */
