/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_GETSID_H__)
#define __HAVE_GETSID_H__


/* do we have or want getsid()? */
#undef HAVE_GETSID /* no */


#endif /* !__HAVE_GETSID_H__ */
