/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_URANDOM_H__)
#define __HAVE_URANDOM_H__


/* do we have /dev/urandom? */
#define HAVE_URANDOM_H  /* yes */


#endif /* !__HAVE_URANDOM_H__ */
