/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_FPOS_H__)
#define __HAVE_FPOS_H__


/* do we have fgetpos & fsetpos functions? */
#undef HAVE_FPOS
#define HAVE_FPOS 1  /* yes */

typedef fpos_t FILEPOS;


#endif /* !__HAVE_FPOS_H__ */
