/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_FPOS_POS_H__)
#define __HAVE_FPOS_POS_H__


/* do we have fgetpos & fsetpos functions? */
#undef HAVE_FPOS_POS	/* no */
#undef FPOS_POS_BITS


#endif /* !__HAVE_FPOS_POS_H__ */
