/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_GETTIME_H__)
#define __HAVE_GETTIME_H__


/* do we have or want clock_gettime()? */
#undef HAVE_GETTIME /* no - no SGI_CYCLE and no REALTIME */


#endif /* !__HAVE_GETTIME_H__ */
