/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__MUST_ALIGN32_H__)
#define __MUST_ALIGN32_H__


/* must we always align 32 bit accesses? */
/* forced to align 32 bit values */
#define MUST_ALIGN32


#endif /* !__MUST_ALIGN32_H__ */
