/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_OFFSCL_H__)
#define __HAVE_OFFSCL_H__


#undef HAVE_OFF_T_SCALAR
#define HAVE_OFF_T_SCALAR /* off_t is a simple value */


#endif /* !__HAVE_OFFSCL_H__ */
