/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__LONGLONG_H__)
#define __LONGLONG_H__


/* do we have/want to use a long long type? */
#define HAVE_LONGLONG
#define LONGLONG_BITS 64  /* yes */

/*
 * how should 64 bit values be formatted?
 *
 * sprintf "%ld" of 0x1234567890123456ULL is -1877855146
 *
 * if defined(L64_FORMAT), ok to use %ld
 * if !defined(L64_FORMAT), use %lld
 */
#undef L64_FORMAT


#endif /* !__LONGLONG_H__ */
