/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__HAVE_STDLIB_H__)
#define __HAVE_STDLIB_H__


/* do we have /usr/include/stdlib.h? */
#define HAVE_STDLIB_H	 /* yes */


#endif /* !__HAVE_STDLIB_H__ */
