/*
 * DO NOT EDIT -- generated by the Makefile
 */


#if !defined(__ARGS_H__)
#define __ARGS_H__


/* what type of variable args do we have? */
#define STDARG /* use <stdarg.h> */
#include <stdarg.h>

/* should we use vsprintf()? */
#define HAVE_VS /* yes */


#endif /* !__ARGS_H__ */
